#include <libintl.h>
#include <math.h>
#include "defaults.h"
#include "string.h"

/* Activate the "Apply" and "Reset" buttons
   (after something has changed, so the user knows
   there is something to either change, or reset back
   to what it was before, be it the default or something
   loaded from a config file */
void activate_buttons()
{
  BUTTON_apply->activate();
  BUTTON_reset->activate();
}

bool status_chng = 0;

void set_change_status(int status)
{
  status_chng = status;
  if (status_chng == 0)
  {
    BUTTON_apply->deactivate();
    BUTTON_reset->deactivate();
  }
  else
  {
    BUTTON_apply->activate();
    BUTTON_reset->activate();
  }
}

void cb_SLIDER_buttonsize(Fl_Slider *, void *)
{
  bool auto_chkbox;
  double val;
  char str[128];

  auto_chkbox = CHECKBUTTON_buttonsizeauto->value();

  if (!auto_chkbox) {
    val = floor(SLIDER_buttonsize->value());
    SLIDER_buttonsize->value(val);

    if (val == 48)
      snprintf(str, sizeof(str), BTN_SIZE_DEFAULT);
    else
      snprintf(str, sizeof(str),  /* Button size option: do overrdide */
               gettext("Button size: %dx%d"), (int) val, (int) val);

    SLIDER_buttonsize->label(strdup(str));
    SLIDER_buttonsize->parent()->redraw();
    SLIDER_buttonsize->activate();
  } else {
    SLIDER_buttonsize->deactivate();
  }

  set_change_status(1);
}

void cb_SLIDER_colorsrows(Fl_Slider *, void *)
{
  double val;
  char str[128];

  val = floor(SLIDER_colorsrows->value());
  SLIDER_colorsrows->value(val);

  if (val == 1)
    snprintf(str, sizeof(str), COLOR_SIZE_DEFAULT);
  else
    snprintf(str, sizeof(str),  /* Color palette size option: do override */
             gettext("Color palette rows: %d"), (int) val);

  SLIDER_colorsrows->label(strdup(str));
  SLIDER_colorsrows->parent()->redraw();

  set_change_status(1);
}

void cb_SLIDER_stampsize(Fl_Slider *, void *)
{
  double val;
  char str[128];

  val = floor(SLIDER_stampsize->value());
  SLIDER_stampsize->value(val);

  if (val == -1)
    snprintf(str, sizeof(str), STAMP_SIZE_DEFAULT);
  else if (val >= 0 && val <= 4)
    snprintf(str, sizeof(str),
             /* Stamp size option: override with a small size */
             gettext("Override: Small (%d)"), (int) val);
  else if (val >= 4 && val < 8)
    snprintf(str, sizeof(str),
             /* Stamp size option: override with a medium size */
             gettext("Override: Medium (%d)"), (int) val);
  else if (val >= 8)
    snprintf(str, sizeof(str),
             /* Stamp size option: override with a large size */
             gettext("Override: Large (%d)"), (int) val);

  SLIDER_stampsize->label(strdup(str));
  SLIDER_stampsize->parent()->redraw();

  set_change_status(1);
}


/* Set defaults on Video/Sound tab */
void DEF_videoandsound()
{
  int diff = 0;

  // Video Defaults

  diff += CHECKBUTTON_fullscreen->value(0);
  diff += CHECKBUTTON_native->value(0);
  diff += CHOICE_windowwidth->value(DEF_WINDOW_WIDTH);
  diff += CHOICE_windowheight->value(DEF_WINDOW_HEIGHT);
  diff += CHECKBUTTON_rotate->value(0);
  diff += CHECKBUTTON_screensaver->value(0);

  // Sound Defaults

  diff += CHECKBUTTON_soundeffects->value(1);
  diff += CHECKBUTTON_stereo->value(1);

  // Interface Size Defaults

  diff += SLIDER_buttonsize->value(48);
  diff += CHECKBUTTON_buttonsizeauto->value(0);
  cb_SLIDER_buttonsize(SLIDER_buttonsize, NULL);
  diff += SLIDER_colorsrows->value(1);

  if (diff)
    activate_buttons();
}

/* Whether "Fullscreen" is checked */
bool CHKBUT_fulls_isdef()
{
  if (CHECKBUTTON_fullscreen->value() == 0)
    return 1;
  else
    return 0;
}

/* Set defaults on Mouse/Keyboard tab */
void DEF_mouseandkeyboard()
{
  int diff = 0;

  // Mouse Defaults

  diff += CHECKBUTTON_fancycursorshapes->value(1);
  diff += CHECKBUTTON_hidecursor->value(0);
  diff += CHECKBUTTON_grabmousepointer->value(0);
  diff += CHECKBUTTON_mousewheelsupport->value(1);
  diff += CHECKBUTTON_nobuttondistinction->value(0);

  // Keyboard Defaults

  diff += CHECKBUTTON_enablekbd->value(1);

  if (diff)
    activate_buttons();
}

bool CHKBUT_fancycursors_isdef()
{
  if (CHECKBUTTON_fancycursorshapes->value() == 1)
    return 1;
  else
    return 0;
}

bool CHKBUT_grabmouse_isdef()
{
  if (CHECKBUTTON_grabmousepointer->value() == 0)
    return 1;
  else
    return 0;
}

bool CHKBUT_mousewheelsup_isdef()
{
  if (CHECKBUTTON_mousewheelsupport->value() == 1)
    return 1;
  else
    return 0;
}

bool CHKBUT_enablekb_isdef()
{
  if (CHECKBUTTON_enablekbd->value() == 1)
    return 1;
  else
    return 0;
}

/* Set defaults on Simplification tab */
void DEF_simplification()
{
  int diff = 0;

  // Interface Simplification Defaults

  diff += CHECKBUTTON_disableshaperotation->value(0);
  diff += CHECKBUTTON_simpleshapeoutlines->value(0);
  diff += CHECKBUTTON_showuppercasetextonly->value(0);

  // Initial Stamp Size Default

  // Control Simplification Defaults
  diff += SLIDER_stampsize->value(-1);
  cb_SLIDER_stampsize(SLIDER_stampsize, NULL);

  diff += CHECKBUTTON_disablequitbutton->value(0);
  diff += CHECKBUTTON_disablestampstool->value(0);
  diff += CHECKBUTTON_disablestampcontrols->value(0);
  diff += CHECKBUTTON_disablestamprotation->value(0);
  diff += CHECKBUTTON_disablemagiccontrols->value(0);
  diff += CHECKBUTTON_disableshapecontrols->value(0);
  diff += CHECKBUTTON_disablelabeltool->value(0);
  diff += CHECKBUTTON_disablebrushspacing->value(0);

  if (diff)
    activate_buttons();
}

bool CHKBUT_norots_isdef()
{
  if (CHECKBUTTON_disableshaperotation->value() == 0)
    return 1;
  else
    return 0;
}

bool CHKBUT_smploutlns_isdef()
{
  if (CHECKBUTTON_simpleshapeoutlines->value() == 0)
    return 1;
  else
    return 0;
}

bool CHKBUT_showUConly_isdef()
{
  if (CHECKBUTTON_showuppercasetextonly->value() == 0)
    return 1;
  else
    return 0;
}

bool CHKBUT_disablequit_isdef()
{
  if (CHECKBUTTON_disablequitbutton->value() == 0)
    return 1;
  else
    return 0;
}

/* Set defaults on Languages tab */
void DEF_locale()
{
  int diff = 0;

  // Locale Defaults

  diff += CHOICE_language->value(0);
  diff += CHECKBUTTON_mirrorstamps->value(0);
  diff += CHECKBUTTON_loadsystemfonts->value(0);
  diff += CHECKBUTTON_alllocalefonts->value(0);

  if (diff)
    activate_buttons();
}

bool CHKBUT_mirstamps_isdef()
{
  if (CHECKBUTTON_mirrorstamps->value() == 0)
    return 1;
  else
    return 0;
}

/* Set defaults on Printing tab */
void DEF_printing()
{
  int diff = 0;

  // Printing Perm. Defaults

  diff += CHECKBUTTON_allowprinting->value(1);
  diff += VALUEINPUT_printdelay->value(0);

  // Print Dialog

  diff += ROUNDBUTTON_printdialogmod->value(1);
  diff += ROUNDBUTTON_printdialogalways->value(0);
  diff += ROUNDBUTTON_printdialognever->value(0);

  // Print Cmd. Defaults

  diff += CHECKBUTTON_usealtprintcmd->value(0);
  TEXTINPUT_printcmd->value("");
  TEXTINPUT_printcmd->deactivate();

  // Print Dialog Cmd. Defaults

  diff += CHECKBUTTON_usealtprintdia->value(0);
  TEXTINPUT_printdia->value("");
  TEXTINPUT_printdia->deactivate();

#ifdef USE_PAPERSIZES
  diff += CHOICE_papersize->value(0);
#endif

  if (diff)
    activate_buttons();
}

bool CHKBUT_allowprint_isdef()
{
  if (CHECKBUTTON_allowprinting->value() == 1)
    return 1;
  else
    return 0;
}

bool VALINP_printdel_isdef()
{
  if (VALUEINPUT_printdelay->value() == 0)
    return 1;
  else
    return 0;
}

/* Set defaults on Saving tab */
void DEF_saving()
{
  int diff = 0;

  // Save Over Earlier Work Defaults

  diff += ROUNDBUTTON_askbeforeoverwriting->value(1);
  diff += ROUNDBUTTON_alwaysoverwriteolderversion->value(0);
  diff += ROUNDBUTTON_alwayssavenewpicture->value(0);

  // Save and Export Directories Defaults

  diff += CHECKBUTTON_usealternatesavedir->value(0);
  FILEINPUT_alternatesavedir->value("");
  FILEINPUT_alternatesavedir->deactivate();
  BUTTON_browsealternatesavedir->deactivate();

  diff += CHECKBUTTON_usealternateexportdir->value(0);
  FILEINPUT_alternateexportdir->value("");
  FILEINPUT_alternateexportdir->deactivate();
  BUTTON_browsealternateexportdir->deactivate();

  // Starting Out Defaults

  diff += CHECKBUTTON_startblank->value(0);
  diff += CHECKBUTTON_newcolorslast->value(0);

  // More Saving Options Defaults

  diff += CHECKBUTTON_disablesavebutton->value(0);
  diff += CHECKBUTTON_autosave->value(0);
  diff += CHECKBUTTON_reversesort->value(0);

  if (diff)
    activate_buttons();
}

/* Set defaults on Data tab */
void DEF_data()
{
  int diff = 0;

  // Data Directory Defaults

  diff += CHECKBUTTON_nolockfile->value(0);

  diff += CHECKBUTTON_usealternatedatadir->value(0);
  FILEINPUT_alternatedatadir->value("");
  FILEINPUT_alternatedatadir->deactivate();
  BUTTON_browsealternatedatadir->deactivate();

  // Colors

  diff += CHECKBUTTON_usecolorfile->value(0);
  FILEINPUT_colorfile->value("");
  FILEINPUT_colorfile->deactivate();
  BUTTON_browsecolorfile->deactivate();

  if (diff)
    activate_buttons();
}

/* Set defaults on Accessibility tab */
void DEF_accessibility()
{
  int diff = 0;
  diff += CHECKBUTTON_mouse_accessibility->value(0);
  diff += CHECKBUTTON_keyboard_mouse->value(0);
  diff += CHECKBUTTON_onscreen_keyboard->value(0);
  diff += CHECKBUTTON_onscreen_keyboard_disable_change->value(0);
  diff += CHOICE_onscreen_keyboard_layout->value(0);

  if (diff)
    activate_buttons();
}

/* Set defaults on Joystick tab */
void DEF_joystick()
{
  int diff = 0;

  if (SPINNER_joystick_dev->value() != 0)
  {
    SPINNER_joystick_dev->value(0);
    diff += 1;
  }
  if (SPINNER_joystick_slowness->value() != 15)
  {
    SPINNER_joystick_slowness->value(15);
    diff += 1;
  }
  if (SPINNER_joystick_threshold->value() != 3200)
  {
    SPINNER_joystick_threshold->value(3200);
    diff += 1;
  }
  if (SPINNER_joystick_maxsteps->value() != 7)
  {
    SPINNER_joystick_maxsteps->value(7);
    diff += 1;
  }
  if (SPINNER_joystick_hat_slowness->value() != 15)
  {
    SPINNER_joystick_hat_slowness->value(15);
    diff += 1;
  }
  if (SPINNER_joystick_hat_timeout->value() != 1000)
  {
    SPINNER_joystick_hat_timeout->value(1000);
    diff += 1;
  }
  if (CHECKBUTTON_joystick_buttons_ignore->value() != 0)
  {
    CHECKBUTTON_joystick_buttons_ignore->value(0);
    diff += 1;
  }
  if (strlen(TEXTINPUT_joystick_buttons_ignore->value()) > 0)
  {
    TEXTINPUT_joystick_buttons_ignore->value("");
    diff += 1;
  }
  TEXTINPUT_joystick_buttons_ignore->deactivate();
  if (SPINNER_joystick_btn_brush->value() != 255)
  {
    SPINNER_joystick_btn_brush->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_label->value() != 255)
  {
    SPINNER_joystick_btn_label->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_new->value() != 255)
  {
    SPINNER_joystick_btn_new->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_print->value() != 255)
  {
    SPINNER_joystick_btn_print->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_shapes->value() != 255)
  {
    SPINNER_joystick_btn_shapes->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_undo->value() != 255)
  {
    SPINNER_joystick_btn_undo->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_eraser->value() != 255)
  {
    SPINNER_joystick_btn_eraser->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_lines->value() != 255)
  {
    SPINNER_joystick_btn_lines->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_open->value() != 255)
  {
    SPINNER_joystick_btn_open->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_redo->value() != 255)
  {
    SPINNER_joystick_btn_redo->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_stamp->value() != 255)
  {
    SPINNER_joystick_btn_stamp->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_escape->value() != 255)
  {
    SPINNER_joystick_btn_escape->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_fill->value() != 255)
  {
    SPINNER_joystick_btn_fill->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_magic->value() != 255)
  {
    SPINNER_joystick_btn_magic->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_pgsetup->value() != 255)
  {
    SPINNER_joystick_btn_pgsetup->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_save->value() != 255)
  {
    SPINNER_joystick_btn_save->value(255);
    diff += 1;
  }
  if (SPINNER_joystick_btn_text->value() != 255)
  {
    SPINNER_joystick_btn_text->value(255);
    diff += 1;
  }

  if (diff)
    activate_buttons();
}

bool RNDBUT_askoverwrite_isdef()
{
  if (ROUNDBUTTON_askbeforeoverwriting->value() == 1)
    return 1;
  else
    return 0;
}

bool CHKBUT_altsavedir_isdef()
{
  if (CHECKBUTTON_usealternatesavedir->value() == 0)
    return 1;
  else
    return 0;
}


bool CHKBUT_altexportdir_isdef()
{
  if (CHECKBUTTON_usealternateexportdir->value() == 0)
    return 1;
  else
    return 0;
}


bool CHKBUT_altdatadir_isdef()
{
  if (CHECKBUTTON_usealternatedatadir->value() == 0)
    return 1;
  else
    return 0;
}


void CB_SetActualTabToDefaults()
{
  if (TABE_videoandsound->visible())
  {
    DEF_videoandsound();
    return;
  }

  if (TABE_mouseandkeyboard->visible())
  {
    DEF_mouseandkeyboard();
    return;
  }

  if (TABE_simplification->visible())
  {
    DEF_simplification();
    return;
  }

  if (TABE_locale->visible())
  {
    DEF_locale();
    return;
  }

  if (TABE_printing->visible())
  {
    DEF_printing();
    return;
  }

  if (TABE_saving->visible())
  {
    DEF_saving();
    return;
  }

  if (TABE_data->visible())
  {
    DEF_data();
    return;
  }
  if (TABE_accessibility->visible())
  {
    DEF_accessibility();
    return;
  }
  if (TABE_joystick->visible())
  {
    DEF_joystick();
    return;
  }
}

/* Set all settings on every tab to their default values */
void CB_SetAllTabsToDefaults()
{
  DEF_videoandsound();
  DEF_mouseandkeyboard();
  DEF_simplification();
  DEF_locale();
  DEF_printing();
  DEF_saving();
  DEF_data();
  DEF_accessibility();
  DEF_joystick();
}
